#ifndef Q2DGRAPHIX_H
#define Q2DGRAPHIX_H

/**
  Qt2DGraphiX Extremely simple and easy 2D graphical framework and wrapper for Qt 4+
  @author AbiusX[.com]
  @version 1.1
  @date 20 december 2010

  */


#include<QDebug>
#include<QSound>
#include"Qt2DGraphiX_engine.h"

QString VersionString="1.1";


namespace Qt2DGraphiX
{

typedef QVector2D Vector2D;
typedef QImage Image2D;

Engine *scr;

enum MouseButtons {NoButton=0,LeftButton=Qt::LeftButton,RightButton=Qt::RightButton,MidButton=Qt::MidButton
        ,LeftRight=0x3,All=0x7};

    void playSound(QString filepath)
    {
        QSound::play(filepath);
    }


    void fillRectangle(QRect r,int c)
    {
        scr->p.fillRect(r,QColor(c));
    }
    void fillRectangle(int x1,int y1, int x2, int y2, int color)
    {
        scr->p.fillRect(x1,y1,x2-x1,y2-y1,QBrush(QColor(color)));
    }
    QSize screenSize()
    {
        return scr->size();
    }


    void MsgBox(QString s)
    {
        QMessageBox::information(scr,"Qt2DGraphiX",s);
    }
    void MsgBox(int num)
    {
        MsgBox(QString("%1").arg(num));
    }
    void MsgBox(double d)
    {
        MsgBox(QString("%1").arg(d));
    }
    void drawCircle(int x,int y, int rx,int ry, int border_color,int fill_color=1)
    {
        if (fill_color==1) fill_color=border_color;
        if (ry==0) ry=rx;
        scr->p.setPen(QColor(border_color));
        scr->p.setBrush(QBrush(QColor(fill_color)));
        scr->p.drawEllipse(x,y,rx,ry);
    }
    void drawCircle(int x,int y, int rx, int border_color,int fill_color=1)
    {
        drawCircle(x,y,rx,rx,border_color,fill_color);
    }
    void drawCircle(Vector2D center,int r,int border_color,int fill_color=1)
    {
        drawCircle(center.x(),center.y(),r,r,border_color,fill_color);
    }

    void putPixel(int x,int y,int color)
    {
        drawCircle(x,y,1,color);
        return;
        QPen p;
        p.setWidth(1);
        p.setColor(color);
        scr->p.setPen(p);
        scr->p.setBrush(QBrush(QColor(color)));
        scr->p.drawPoint(x,y);

    }
    void putPixel(QPoint p,int color)
    {
        putPixel(p.x(),p.y(),color);
    }
    void putPixel(Vector2D p,int color)
    {
        putPixel(p.toPoint(),color);
    }
    void drawLine(int x1,int y1,int x2,int y2,int color)
    {
        scr->p.setPen(QColor(color));
        scr->p.drawLine(x1,y1,x2,y2);
    }
    void drawLine(Vector2D p1,Vector2D p2,int color)
    {
        drawLine(p1.x(),p1.y(),p2.x(),p2.y(),color);
    }


    MouseButtons getMousePressButton()
    {
        int x=scr->mousePressButton;
        scr->mousePressButton=0;
        return MouseButtons(x);
    }
    MouseButtons getMouseDownButton()
    {
        int x=scr->mouseDownButton;
        scr->mouseDownButton=0;
        return MouseButtons(x);
    }
    Vector2D getMouseCoords()
    {
        return Vector2D(scr->mouseCoords);
    }
    int getKeyboardKey()
    {

        int x=scr->keyboardKey;
        scr->keyboardKey=0;
        return x;
    }

    void setTransform(float translate_x=0,float translate_y=0,float rotate=0,float scale_x=1,float scale_y=1)
    {
        QTransform tr;
        tr.translate(translate_x,translate_y);
        tr.rotate(rotate);
        tr.scale(scale_x,scale_y);
        scr->p.setTransform(tr);
    }
    Image2D loadImage(QString filepath)
    {
        QImage img(filepath);
        return img;
    }
    bool isImage(Image2D img)
    {
        return !img.isNull();
    }
    void drawImage(int x,int y,Image2D img,int width=0,int height=0)
    {
        scr->p.drawImage(x,y,img,0,0,width,height);
    }
    void putText(int x,int y,QString text,int color)
    {
        scr->p.setPen(QColor(color));
        scr->p.drawText(x,y,text);
    }
    int getFrameCount()
    {
        return scr->frameCounter;
    }
    int getTime()
    {
        return scr->timeCounter;
    }



    int init(int argc,char *argv[],void (*setup)(),int (*hook)(),QString title,int width=640,int height=480,int FPS=60)
    {
        QApplication a(argc,argv);
        //        qAddPostRoutine(cleanup);
        a.setApplicationName("Qt2DGraphiX");
        a.setOrganizationName("AbiusX");
        a.setOrganizationDomain("AbiusX.com");
        a.setApplicationVersion(VersionString);
        setup();
        Engine x(hook,title,FPS,width,height);
        scr=&x;
        scr->show();
        return a.exec();
    }

}



#endif Q2DGRAPHIX_H
