#ifndef QT2DGRAPHIX_H
#define QT2DGRAPHIX_H


#include <QDialog>
#include <QVector2D>
#include <QGLWidget>
#include <QTimer>
#include <QDebug>
#include <QKeyEvent>
#include <QMouseEvent>
#include <QMessageBox>
#include <QApplication>

namespace Qt2DGraphiX
{
class Engine : public QGLWidget {
    Q_OBJECT
public:
    QPoint mouseCoords;
    int mousePressButton;
    int mouseDownButton;
    int keyboardKey;
    void initEvents()
    {
        mouseDownButton=0;
        mousePressButton=0;
        mouseCoords=QPoint(0,0);
        keyboardKey=0;
    }


    int (*update)();
    QPainter p;
    int frameCounter;
    int timeCounter;
    int timerInterval;
    QGLWidget *screen;
    QTimer *timer;
    Engine(int (*hook)()=0,QString title="",int FPS=60,int width=640,int height=480,QGLWidget *parent=0) :
            QGLWidget(parent)
            ,update(hook)
            ,frameCounter(0)
            ,timeCounter(0)
            ,timerInterval(1000/FPS)
    {
        screen=this;
//        screen->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed));
        screen->resize(width,height);
        if (title=="") title="Qt2DGraphiX Application";
        screen->setWindowTitle(title);
        timer = new QTimer(this);
        connect(timer, SIGNAL(timeout()), this, SLOT(frame()));
        timer->start(timerInterval);

        //This enabled MouseMove event on QGLWidget of ours
        screen->setMouseTracking(true);
        //Binding QGLWidget's MouseMove event to us
        //screen->installEventFilter(this);
        //Hiding cursor on QGLWidget
        screen->setCursor(Qt::BlankCursor);

        initEvents();
    }

    ~Engine()
    {

    }

protected:

    /**
      Here we receive all events that occur on screen (QGLWidget) and handle them
      */
//    bool eventFilter(QObject *sender, QEvent *e)
//    {
//        //If it's the QGLWidget's MouseMove, handle it as our MouseMove
//        if (e->type()==QEvent::MouseMove)
//        {
//            mouseMoveEvent((QMouseEvent *)e);
//            return true;
//        }
//        else
//            //Handle all the other events by Qt!
//            return QObject::eventFilter(sender,e);
//    }
    /**
      When the mouse is clicked on Starting state, it should shoot the ball
      */
    void mousePressEvent(QMouseEvent *e)
    {
        mousePressButton=e->button();
    }
    /**
      When the game window is resized, the game area should be resize appropriately
      */
    void resizeEvent(QResizeEvent *e)
    {
        e=e;
    }
    /**
      Keyboard handling
      */
    void keyPressEvent(QKeyEvent *e)
    {
        if (e->key()==Qt::Key_Escape)
            this->close();
        keyboardKey=e->key();
    }
    /**
      Mousemove and pad handling
      */
    void mouseMoveEvent(QMouseEvent *e)
    {
        mouseCoords=e->pos();
        mouseDownButton=e->buttons();
    }

private:




public slots:
    int demo()
    {
        static QPoint po(10,20);
        p.fillRect(screen->geometry(),qRgb(255,255,255));
        p.drawText(po,"Welcome to Qt2DGraphiX by AbiusX[.com]!");
        po+=QPoint(1,1);
        po.setX(po.x()%screen->width());
        po.setY(po.y()%screen->height());
        return 0;
    }

    void frame()
    {
        //Drawing Graphics
        timeCounter+=timer->interval();
        if (p.begin(screen))
        {
            frameCounter++;
            if (update==0) demo();
            else update();
            p.end();
        }


    }
};

}

#endif QT2DGRAPHIX_H
